#!/bin/sh --
eval '(exit $?0)' && eval 'exec perl -w -x -S $0 ${1+"$@"}' && eval 'exec perl 
if 0; # the 1st 4 lines make compatible with kernel/perl/bash/ash/sh/csh
#!perl -w
# epstopdf.pl
# original Sebastian Rahtz, for Elsevier Science
# change by Thomas Esser, Sept. 1998
# BB (atend) change by pts@fazekas.hu at Sun Oct 29 00:11:24 CEST 2000
use integer;
use strict;

# Change by Thomas Esser, Sept. 1998: The above lines allows us to find
# perl along $PATH rather than guessing a fixed location. The above
# construction should work with most shells.

# A script to transform an EPS file so that:
#   a) it is guarenteed to start at the 0,0 coordinate
#   b) it sets a page size exactly corresponding to the BoundingBox
# This means that when Ghostscript renders it, the result needs no
# cropping, and the PDF MediaBox is correct.
#   c) the result is piped to Ghostscript and a PDF version written
#
# It needs a Level 2 PS interpreter.
# If the bounding box is not right, of course, you have problems...
#
# The only thing I have not allowed for is the case of
# "%%BoundingBox: (atend)", which is more complicated.
#
# Sebastian Rahtz, for Elsevier Science
#
# now with extra tricks from Hans Hagen's texutil.
#
use Getopt::Long ;
my $filedate="1998/10/22";
my $fileversion="2.4";
my $GS;
if ($^O eq 'MSWin32')  { $GS="gswin32c";}
else                   { $GS="gs"; }
   
my $opt_debug=0;
my $opt_compress=1;
my $opt_gs=1;
my $opt_outfile="";
my $result = GetOptions (
 "debug!",
 "compress!",
 "gs!",
 "outfile=s",
);
my $Filename=$ARGV[0];
my $GSOPTS;
if (!$opt_compress)
  { $GSOPTS="-dUseFlateCompression=false " ;}
else 
  { $GSOPTS=""; }

open(TMP,"< $Filename") or  die "Usage: $0 <filename>\n";
binmode TMP;
$_=$Filename;
s/\.[A-Za-z]*$//;
if ($opt_gs) {
   open(OUT,"| $GS -q -dBATCH -sDEVICE=pdfwrite $GSOPTS -sOutputFile='$_.pdf' -") 
   # open(OUT,">t.o")
   or  die "cannot open Ghostscript for piped input";
   if ($opt_debug) { print STDERR "read $Filename, write $_.pdf\n"; }
}
else
 { 
   if ($opt_outfile ne "")
     { open(OUT,">$opt_outfile") or die "cannot write $opt_outfile" ; 
      if ($opt_debug) { print STDERR "read $Filename, write $opt_outfile\n"; }
     }
   else { open(OUT,">&STDOUT"); }
}

my $bbneeded=1;
my $bbpatt="[0-9\.\-]";
my $bbxpatt = "[0-9eE\.\-]";
my $bbre="%%(?:HiRes|Exact)?BoundingBox:\\s*($bbpatt+)\\s+($bbpatt+)\\s+($bbpatt+)\\s+($bbpatt+)"; # by pts@fazekas.hu
while (<TMP>) { 
AGAIN:
#
# throw away binary junk before %!PS
# by pts@fazekas.hu: binary junk longer than 1 line??
# by pts@fazekas.hu: ninary junk containing %%BoundingBox?? (not very probable)
#
 if (/%!PS-Adobe/)
          { s/(.*)%!PS-Adobe/%!PS-Adobe/m ;
            print OUT; }
#
# locate BoundingBox
#
 elsif (/%%BoundingBox: \(atend\)/) {

	# by pts@fazekas.hu
	# What You Ask Is What You Get. But slowly.
	# Imp: really calculate hiresbb
	my $L;
	die unless open S, "< $Filename";
	die unless $L=sysseek S, 0, 2; # normal seek() always returns 1.
	$L-=4096; $L=0 if $L<0;
	die unless sysseek S, $L, 0;
	die unless sysread S, $L, 4096;
	close S;
	if ($L=~/($bbre)/o) {
	  my $V=$1;
	  # die $V;
	  s/%%BoundingBox: \(atend\)/$V/;
	  goto AGAIN;
	}
	print STDERR "$_ could not be resolved in line $. Giving up.\n";
	$bbneeded = 0;
        print OUT;
	}
 elsif (/$bbre/o ) {
     # only read the *first* bounding box
     # imp convert hires to lores
     if ($bbneeded) {
	 if ($opt_debug) { print STDERR "old BB is $1 $2 $3 $4\n";}
         
	 my $width = $3 - $1;
	 my $height = $4 - $2;
	 my $xoffset = 0 - $1;
	 my $yoffset = 0 - $2;
	 if ($opt_debug) {
	     print STDERR "new BB is 0 0 $width $height, offset $xoffset $yoffset\n";
	 }
	 print OUT "%%BoundingBox: 0 0 $width $height\n";
	 print OUT "<< /PageSize [$width $height] >> setpagedevice\n";
	 print OUT "gsave $xoffset $yoffset translate\n";
	 $bbneeded=0;
     }

# else ignore that embedded BoundingBox anyway
 }
  elsif (/%%(HiResB|ExactB)oundingBox:\s*($bbxpatt+)\s+($bbxpatt+)\s+($bbxpatt+)\s+($bbxpatt+)/ ) { 
     print STDERR "BBox already calculated. Bbox $_ in line $. not used.\n";
 }
else
 {    print OUT;  }
}
close(TMP);
print OUT "grestore\n";
close(OUT);

# end of epstopdf.pl
